package com.mapmyindia.mapmyindiagl.api;

import com.mapbox.geojson.Point;
import com.mmi.services.api.alongroute.MapmyIndiaPOIAlongRoute;
import com.mmi.services.api.autosuggest.MapmyIndiaAutoSuggest;
import com.mmi.services.api.directions.MapmyIndiaDirections;
import com.mmi.services.api.distance.MapmyIndiaDistanceMatrix;
import com.mmi.services.api.eloc.MapmyIndiaELoc;
import com.mmi.services.api.feedback.MapmyIndiaFeedback;
import com.mmi.services.api.geocoding.MapmyIndiaGeoCoding;
import com.mmi.services.api.nearby.MapmyIndiaNearby;
import com.mmi.services.api.placedetail.MapmyIndiaPlaceDetail;
import com.mmi.services.api.reversegeocode.MapmyIndiaReverseGeoCode;
import com.mmi.services.api.textsearch.MapmyIndiaTextSearch;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * * Created by Saksham on 14-05-2021.
 **/
public class RestApiHelper {

    static MapmyIndiaAutoSuggest autoSuggest(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaAutoSuggest.Builder builder = MapmyIndiaAutoSuggest.builder()
                .query(toString(argument.get("query")));
        if (argument.containsKey("filter")) {
            builder.filter(toString(argument.get("filter")));
        }
        if (argument.containsKey("bridge")) {
            if (toBoolean(argument.get("bridge"))) {
                builder.bridge(true);
            }
        }
        if (argument.containsKey("pod")) {
            builder.pod(toString(argument.get("pod")));
        }
        if (argument.containsKey("location")) {
            final List<?> data = toList(argument.get("location"));
            builder.setLocation(toDouble(data.get(0)), toDouble(data.get(1)));
        }
        if (argument.containsKey("tokenizeAddress")) {
            if (toBoolean(argument.get("tokenizeAddress"))) {
                builder.tokenizeAddress(true);
            }
        }
        if (argument.containsKey("zoom")) {
            builder.zoom(toDouble(argument.get("zoom")));
        }
        return builder.build();
    }

    static MapmyIndiaGeoCoding geoCoding(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaGeoCoding.Builder builder = MapmyIndiaGeoCoding.builder()
                .setAddress(toString(argument.get("address")));
        if (argument.containsKey("bias")) {
            builder.bias(toInt(argument.get("bias")));
        }
        if (argument.containsKey("bound")) {
            builder.bound(toString(argument.get("bound")));
        }
        if (argument.containsKey("itemCount")) {
            builder.itemCount(toInt(argument.get("itemCount")));
        }
        if (argument.containsKey("podFilter")) {
            builder.podFilter(toString(argument.get("podFilter")));
        }
        return builder.build();
    }

    static MapmyIndiaReverseGeoCode reverseGeoCode(Object o) {
        Map<?, ?> argument = toMap(o);
        final List<?> data = toList(argument.get("location"));
        return MapmyIndiaReverseGeoCode.builder()
                .setLocation(toDouble(data.get(0)), toDouble(data.get(1)))
                .build();

    }

    static MapmyIndiaNearby nearby(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaNearby.Builder builder = MapmyIndiaNearby.builder()
                .keyword(toString(argument.get("keyword")));
        if (argument.containsKey("location")) {
            final List<?> data = toList(argument.get("location"));
            builder.setLocation(toDouble(data.get(0)), toDouble(data.get(1)));
        }
        if (argument.containsKey("eLoc")) {
            builder.setLocation(toString(argument.get("eLoc")));
        }
        if (argument.containsKey("page")) {
            builder.page(toInt(argument.get("page")));
        }
        if (argument.containsKey("radius")) {
            builder.radius(toInt(argument.get("radius")));
        }
        if (argument.containsKey("pod")) {
            builder.pod(toString(argument.get("pod")));
        }
        if (argument.containsKey("bounds")) {
            builder.bounds(toString(argument.get("bounds")));
        }
        if (argument.containsKey("filter")) {
            builder.filter(toString(argument.get("filter")));
        }
        if (argument.containsKey("explain")) {
            if (toBoolean(argument.get("explain"))) {
                builder.explain(true);
            }
        }
        if (argument.containsKey("richData")) {
            if (toBoolean(argument.get("richData"))) {
                builder.richData(true);
            }
        }
        if (argument.containsKey("searchBy")) {
            builder.searchBy(toString(argument.get("searchBy")));
        }
        if (argument.containsKey("sortBy")) {
            builder.sortBy(toString(argument.get("sortBy")));
        }
        if (argument.containsKey("userName")) {
            builder.userName(toString(argument.get("userName")));
        }

        return builder.build();
    }

    static MapmyIndiaDirections directions(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaDirections.Builder builder = MapmyIndiaDirections.builder()
                .profile(toString(argument.get("profile")))
                .resource(toString(argument.get("resource")))
                .overview(toString(argument.get("overview")))
                .geometries(toString(argument.get("geometries")));
        if (argument.containsKey("originPoint")) {
            final List<?> data = toList(argument.get("originPoint"));
            builder.origin(Point.fromLngLat(toDouble(data.get(1)), toDouble(data.get(0))));
        }
        if (argument.containsKey("destinationPoint")) {
            final List<?> data = toList(argument.get("destinationPoint"));
            builder.destination(Point.fromLngLat(toDouble(data.get(1)), toDouble(data.get(0))));
        }
        if (argument.containsKey("originELoc")) {
            builder.origin(toString(argument.get("originELoc")));
        }
        if (argument.containsKey("destinationELoc")) {
            builder.destination(toString(argument.get("destinationELoc")));
        }
        if (argument.containsKey("waypointPoint")) {
            final List<?> data = toList(argument.get("waypointPoint"));
            for (int i = 0; i < data.size(); i++) {
                final List<?> location = toList(data.get(i));
                builder.addWaypoint(Point.fromLngLat(toDouble(location.get(1)), toDouble(location.get(0))));
            }
        }
        if (argument.containsKey("waypointELoc")) {
            final List<?> data = toList(argument.get("waypointELoc"));
            for (int i = 0; i < data.size(); i++) {
                builder.addWaypoint(toString(data.get(i)));
            }
        }
        if (argument.containsKey("excludes")) {
            List<?> excludesArray = toList(argument.get("excludes"));
            String[] excludes = new String[excludesArray.size()];
            for (int i = 0; i < excludesArray.size(); i++) {
                excludes[i] = toString(excludesArray.get(i));
            }
            builder.excludes(excludes);
        }
        if (argument.containsKey("annotations")) {
            List<?> annotationsArray = toList(argument.get("annotations"));
            String[] annotations = new String[annotationsArray.size()];
            for (int i = 0; i < annotationsArray.size(); i++) {
                annotations[i] = toString(annotationsArray.get(i));
            }
            builder.annotations(annotations);
        }
        if (argument.containsKey("routeType")) {
            builder.routeType(toInt(argument.get("routeType")));
        }
        if (argument.containsKey("steps")) {
            builder.steps(toBoolean(argument.get("steps")));
        }
        if (argument.containsKey("alternatives")) {
            builder.alternatives(toBoolean(argument.get("alternatives")));
        }
        if (argument.containsKey("approaches")) {
            List<?> approachesArray = toList(argument.get("approaches"));
            String[] approaches = new String[approachesArray.size()];
            for (int i = 0; i < approachesArray.size(); i++) {
                approaches[i] = toString(approachesArray.get(i));
            }
            builder.addApproaches(approaches);
        }
        if (argument.containsKey("bearing")) {
            Map<?, ?> bearing = toMap(argument.get("bearing"));
            if (bearing.containsKey("angle") && bearing.containsKey("tolerance")) {
                builder.addBearing(toDouble(bearing.get("angle")), toDouble(bearing.get("tolerance")));
            }
        }
        if (argument.containsKey("waypointIndices")) {
            List<?> waypointIndicesArray = toList(argument.get("waypointIndices"));
            Integer[] waypointIndices = new Integer[waypointIndicesArray.size()];
            for (int i = 0; i < waypointIndicesArray.size(); i++) {
                waypointIndices[i] = toInt(waypointIndicesArray.get(i));
            }
            builder.addWaypointIndices(waypointIndices);
        }
        if (argument.containsKey("waypointNames")) {
            List<?> waypointNamesArray = toList(argument.get("waypointNames"));
            String[] waypointNames = new String[waypointNamesArray.size()];
            for (int i = 0; i < waypointNamesArray.size(); i++) {
                waypointNames[i] = toString(waypointNamesArray.get(i));
            }
            builder.addWaypointNames(waypointNames);
        }
        if (argument.containsKey("waypointTargets")) {
            List<?> waypointTargetsArray = toList(argument.get("waypointTargets"));
            Point[] waypointTargets = new Point[waypointTargetsArray.size()];
            for (int i = 0; i < waypointTargetsArray.size(); i++) {
                List<?> waypointTargetsPoint = toList(waypointTargetsArray.get(i));
                waypointTargets[i] = Point.fromLngLat(toDouble(waypointTargetsPoint.get(1)), toDouble(waypointTargetsPoint.get(0)));

            }
            builder.addWaypointTargets(waypointTargets);
        }
        if (argument.containsKey("bannerInstructions")) {
            builder.bannerInstructions(toBoolean(argument.get("bannerInstructions")));
        }
        if (argument.containsKey("isSort")) {
            builder.isSort(toBoolean(argument.get("isSort")));
        }
        if (argument.containsKey("lessVerbose")) {
            builder.lessVerbose(toBoolean(argument.get("lessVerbose")));
        }
        if (argument.containsKey("radiuses")) {
            List<?> radiusesArray = toList(argument.get("radiuses"));
            double[] radiuses = new double[radiusesArray.size()];
            for (int i = 0; i < radiusesArray.size(); i++) {
                radiuses[i] = toDouble(radiusesArray.get(i));
            }
            builder.radiuses(radiuses);
        }
        if (argument.containsKey("routeRefresh")) {
            builder.routeRefresh(toBoolean(argument.get("routeRefresh")));
        }
        if (argument.containsKey("deviceId")) {
            builder.deviceId(toString(argument.get("deviceId")));
        }
        if (argument.containsKey("roundaboutExits")) {
            builder.roundaboutExits(toBoolean(argument.get("roundaboutExits")));
        }
        if (argument.containsKey("sessionId")) {
            builder.sessionId(toString(argument.get("sessionId")));
        }
        if (argument.containsKey("voiceInstructions")) {
            builder.voiceInstructions(toBoolean(argument.get("voiceInstructions")));
        }
        if (argument.containsKey("voiceUnits")) {
            builder.voiceUnits(toString(argument.get("voiceUnits")));
        }

        return builder.build();
    }

    static MapmyIndiaDistanceMatrix distanceMatrix(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaDistanceMatrix.Builder builder = MapmyIndiaDistanceMatrix.builder()
                .profile(toString(argument.get("profile")))
                .resource(toString(argument.get("resource")));
        if (argument.containsKey("coordinates")) {
            List<?> list = toList(argument.get("coordinates"));
            List<String> eLocList = new ArrayList<>();
            for (Object element : list) {
                eLocList.add(toString(element));
            }
            builder.coordinateList(eLocList);
        }
        if (argument.containsKey("sources")) {
            List<?> list = toList(argument.get("sources"));
            List<Integer> sources = new ArrayList<>();
            for (Object element : list) {
                sources.add(toInt(element));
            }
            builder.sources(sources);
        }
        if (argument.containsKey("destinations")) {
            List<?> list = toList(argument.get("destinations"));
            List<Integer> sources = new ArrayList<>();
            for (Object element : list) {
                sources.add(toInt(element));
            }
            builder.destinations(sources);
        }
        if (argument.containsKey("routeType")) {
            builder.routeType(toInt(argument.get("routeType")));
        }
        return builder.build();

    }

    static MapmyIndiaFeedback feedback(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaFeedback.Builder builder = MapmyIndiaFeedback.builder()
                .typedKeyword(toString(argument.get("typedKeyword")))
                .eLoc(toString(argument.get("eLoc")))
                .index(toInt(argument.get("index")))
                .appVersion(toString(argument.get("appVersion")));

        if (argument.containsKey("location")) {
            List<?> list = toList(argument.get("location"));
            builder.latitude(toDouble(list.get(0)));
            builder.longitude(toDouble(list.get(1)));
        }
        if (argument.containsKey("locationName")) {
            builder.locationName(toString(argument.get("locationName")));
        }
        if (argument.containsKey("userName")) {
            builder.userName(toString(argument.get("userName")));
        }

        return builder.build();
    }

    static MapmyIndiaPOIAlongRoute poiAlongRoute(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaPOIAlongRoute.Builder builder = MapmyIndiaPOIAlongRoute.builder()
                .path(toString(argument.get("path")))
                .category(toString(argument.get("category")))
                .geometries(toString(argument.get("geometries")));

        if (argument.containsKey("buffer")) {
            builder.buffer(toInt(argument.get("buffer")));
        }
        if (argument.containsKey("page")) {
            builder.page(toInt(argument.get("page")));
        }
        if (argument.containsKey("sort")) {
            builder.sort(toBoolean(argument.get("sort")));
        }

        return builder.build();
    }

    static MapmyIndiaTextSearch textSearch(Object o) {
        Map<?, ?> argument = toMap(o);
        MapmyIndiaTextSearch.Builder builder = MapmyIndiaTextSearch.builder()
                .query(toString(argument.get("query")));

        if (argument.containsKey("filter")) {
            builder.filter(toString(argument.get("filter")));
        }
        if (argument.containsKey("username")) {
            builder.username(toString(argument.get("username")));
        }
        if (argument.containsKey("explain")) {
            if (toBoolean(argument.get("explain"))) {
                builder.explain(true);
            }
        }

        if (argument.containsKey("bridge")) {
            if (toBoolean(argument.get("bridge"))) {
                builder.bridge(true);
            }
        }

        if (argument.containsKey("location")) {
            final List<?> data = toList(argument.get("location"));
            builder.setLocation(toDouble(data.get(0)), toDouble(data.get(1)));
        }

        return builder.build();
    }
    
    static MapmyIndiaPlaceDetail placeDetail(Object o) {
        Map<?, ?> argument = toMap(o);
        return MapmyIndiaPlaceDetail.builder()
                .eLoc(toString(argument.get("eLoc")))
                .build();
    }
    
    static MapmyIndiaELoc placeDetailLegacy(Object o) {
        Map<?, ?> argument = toMap(o);
        return MapmyIndiaELoc.builder()
                .setELoc(toString(argument.get("eLoc")))
                .build();
    }

    private static Map<?, ?> toMap(Object o) {
        return (Map<?, ?>) o;
    }

    private static List<?> toList(Object o) {
        return (List<?>) o;
    }

    private static double toDouble(Object o) {
        return ((Number) o).doubleValue();
    }

    private static float toFloat(Object o) {
        return ((Number) o).floatValue();
    }

    private static int toInt(Object o) {
        return ((Number) o).intValue();
    }

    private static boolean toBoolean(Object o) {
        return (boolean) o;
    }

    private static String toString(Object o) {
        return (String) o;
    }

}
