package com.mapmyindia.mapmyindiagl.api;

import androidx.annotation.NonNull;

import com.google.gson.Gson;
import com.mmi.services.api.PlaceResponse;
import com.mmi.services.api.alongroute.models.POIAlongRouteResponse;
import com.mmi.services.api.autosuggest.model.AutoSuggestAtlasResponse;
import com.mmi.services.api.directions.models.DirectionsResponse;
import com.mmi.services.api.distance.models.DistanceResponse;
import com.mmi.services.api.geocoding.GeoCodeResponse;
import com.mmi.services.api.nearby.model.NearbyAtlasResponse;
import com.mmi.services.api.placedetail.model.PlaceDetailResponse;

import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * * Created by Saksham on 14-05-2021.
 **/
public class RestApiMethodHandler implements MethodChannel.MethodCallHandler {


    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        switch (call.method) {
            case "restApi#autosuggest":
                RestApiHelper.autoSuggest(call.arguments)
                        .enqueueCall(new Callback<AutoSuggestAtlasResponse>() {
                            @Override
                            public void onResponse(Call<AutoSuggestAtlasResponse> call, Response<AutoSuggestAtlasResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<AutoSuggestAtlasResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#geocode":
                RestApiHelper.geoCoding(call.arguments)
                        .enqueueCall(new Callback<GeoCodeResponse>() {
                            @Override
                            public void onResponse(Call<GeoCodeResponse> call, Response<GeoCodeResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<GeoCodeResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#revgeocode":
                RestApiHelper.reverseGeoCode(call.arguments)
                        .enqueueCall(new Callback<PlaceResponse>() {
                            @Override
                            public void onResponse(Call<PlaceResponse> call, Response<PlaceResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<PlaceResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#nearby":
                RestApiHelper.nearby(call.arguments)
                        .enqueueCall(new Callback<NearbyAtlasResponse>() {
                            @Override
                            public void onResponse(Call<NearbyAtlasResponse> call, Response<NearbyAtlasResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<NearbyAtlasResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#direction":
                RestApiHelper.directions(call.arguments)
                        .enqueueCall(new Callback<DirectionsResponse>() {
                            @Override
                            public void onResponse(Call<DirectionsResponse> call, Response<DirectionsResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? response.body().toJson() : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<DirectionsResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#distancematrix":
                RestApiHelper.distanceMatrix(call.arguments)
                        .enqueueCall(new Callback<DistanceResponse>() {
                            @Override
                            public void onResponse(Call<DistanceResponse> call, Response<DistanceResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<DistanceResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#feedback":
                RestApiHelper.feedback(call.arguments)
                        .enqueueCall(new Callback<Void>() {
                            @Override
                            public void onResponse(Call<Void> call, Response<Void> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.code());
                                } else {
                                    result.error(response.code() + "", response.headers().get("message"), null);
                                }
                            }

                            @Override
                            public void onFailure(Call<Void> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#poialongroute":
                RestApiHelper.poiAlongRoute(call.arguments)
                        .enqueueCall(new Callback<POIAlongRouteResponse>() {
                            @Override
                            public void onResponse(Call<POIAlongRouteResponse> call, Response<POIAlongRouteResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<POIAlongRouteResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#textsearch":
                RestApiHelper.textSearch(call.arguments)
                        .enqueueCall(new Callback<AutoSuggestAtlasResponse>() {
                            @Override
                            public void onResponse(Call<AutoSuggestAtlasResponse> call, Response<AutoSuggestAtlasResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<AutoSuggestAtlasResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#placedetail":
                RestApiHelper.placeDetail(call.arguments)
                        .enqueueCall(new Callback<PlaceDetailResponse>() {
                            @Override
                            public void onResponse(Call<PlaceDetailResponse> call, Response<PlaceDetailResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<PlaceDetailResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;

            case "rest#placedetaillegacy":
                RestApiHelper.placeDetailLegacy(call.arguments)
                        .enqueueCall(new Callback<PlaceResponse>() {
                            @Override
                            public void onResponse(Call<PlaceResponse> call, Response<PlaceResponse> response) {
                                if (response.isSuccessful()) {
                                    result.success(response.body() != null ? new Gson().toJson(response.body()) : null);
                                } else {
                                    result.error(response.code() + "", response.message(), response.body());
                                }
                            }

                            @Override
                            public void onFailure(Call<PlaceResponse> call, Throwable t) {
                                result.error("-1", t.getMessage(), null);
                            }
                        });
                break;
        }
    }

}
