part of mapmyindia_gl;

class DirectionResponse {
  final String? code;
  final String? message;
  final String? routeId;
  final String? sessionId;
  final List<DirectionsWaypoint>? waypoints;
  final List<DirectionsRoute>? routes;

  DirectionResponse({this.code, this.message, this.routeId, this.sessionId, this.waypoints, this.routes});

  static DirectionResponse fromMap(Map<String, dynamic> map) {
    List<DirectionsWaypoint>? waypoints;
    if(map.containsKey('waypoints')) {
      List<dynamic> waypointMapList = map['waypoints'] as List<dynamic>;
      waypoints = List.generate(waypointMapList.length, (index)  {
        return DirectionsWaypoint.fromMap(waypointMapList[index] as Map<String, dynamic>);
      });
    }
    List<DirectionsRoute>? directionRoutes;
    if(map.containsKey('routes')) {
      List<dynamic> routes = map['routes'] as List<dynamic>;
      directionRoutes = List.generate(routes.length, (index)  {
        return DirectionsRoute.fromJson(routes[index] as Map<String, dynamic>);
      });
    }
    return DirectionResponse(
      code: map["code"] as String?,
      message: map["message"] as String?,
      routeId: map['routeId'] as String?,
      sessionId: map['sessionId'] as String?,
      waypoints: waypoints,
      routes: directionRoutes,
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    List<Map<String, dynamic>>? waypointMap;
    if(waypoints != null) {
      waypointMap = List.generate(waypoints?.length == null? 0: waypoints!.length, (index) {
        return waypoints![index].toMap();
      });
    }
    List<Map<String, dynamic>>? routeMap;
    if(routes != null) {
      routeMap = List.generate(routes?.length == null? 0: routes!.length, (index) {
        return routes![index].toMap();
      });
    }

    addIfNonNull('code', this.code);
    addIfNonNull('message', this.message);
    addIfNonNull('routeId', this.routeId);
    addIfNonNull('sessionId', this.sessionId);
    addIfNonNull('waypoints', waypointMap);
    addIfNonNull('routes', routeMap);
    return optionsMap;
  }
}


class DirectionsWaypoint {
  final String? name;
  final LatLng? location;

  DirectionsWaypoint({this.name, this.location});

  static DirectionsWaypoint fromMap(Map<String, dynamic> map) {
    LatLng? location;
    if(map.containsKey('location')) {
      List<dynamic> points = map['location'];
      location = LatLng(points[1], points[0]);
    }
    return DirectionsWaypoint(
        name: map['name'] as String?,
        location: location
    );
  }

  Map<String, dynamic> toMap() => {
    'name': this.name,
    'location': this.location?.toJson()
  };
}

class DirectionsRoute {
  final double? distance;
  final double? duration;
  final String? geometry;
  final double? weight;
  final String? weightName;
  final int? routeIndex;
  final List<RouteLeg>? legs;
  final RouteOptions? routeOptions;

  DirectionsRoute(
      {this.distance,
        this.duration,
        this.geometry,
        this.weight,
        this.weightName,
        this.routeIndex,
        this.legs,
        this.routeOptions});

  static DirectionsRoute fromJson(Map<String, dynamic> map) {
    List<RouteLeg>? legs;
    if(map.containsKey('legs') && map['legs'] != null) {
      List<dynamic> legsMap = map['legs'] as List<dynamic>;
      legs = List.generate(legsMap.length, (index)  {
        return RouteLeg.fromMap(legsMap[index] as Map<String, dynamic>);
      });
    }
    RouteOptions? routeOptions;
    if(map.containsKey('routeOptions') && map['routeOptions'] != null) {
      routeOptions = RouteOptions.fromJson(map['routeOptions']);
    }
    return DirectionsRoute(
        distance:(map["distance"])?.toDouble(),
        duration: (map['duration'])?.toDouble(),
        geometry: map['geometry'],
        weight: (map['weight'])?.toDouble(),
        weightName: map['weight_name'],
        routeIndex: map['routeIndex'],
        legs: legs,
        routeOptions: routeOptions
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    List<Map<String, dynamic>>? legsMap;
    if(legs != null) {
      legsMap = List.generate(legs?.length == null? 0: legs!.length, (index) {
        return legs![index].toMap();
      });
    }
    addIfNonNull('distance', this.distance);
    addIfNonNull('duration', this.duration);
    addIfNonNull('geometry', this.geometry);
    addIfNonNull('weight', this.weight);
    addIfNonNull('weight_name', this.weightName);
    addIfNonNull('routeIndex', this.routeIndex);
    addIfNonNull('legs', legsMap);
    addIfNonNull('routeOptions', this.routeOptions?.toMap());
    return optionsMap;
  }

}

class RouteLeg {
  final double? distance;
  final double? duration;
  final String? summary;
  final List<LegStep>? steps;
  final LegAnnotation? annotation;

  RouteLeg({this.distance, this.duration, this.summary, this.steps, this.annotation});

  static RouteLeg fromMap(Map<String, dynamic> map) {

    List<LegStep>? steps;
    if(map.containsKey('steps')) {
      List<dynamic> stepsMap = map['steps'] as List<dynamic>;
      steps = List.generate(stepsMap.length, (index)  {
        return LegStep.fromMap(stepsMap[index] as Map<String, dynamic>);
      });
    }
    LegAnnotation? annotation;
    if(map.containsKey('annotation')) {
      annotation = LegAnnotation.fromMap(map['annotation']);
    }

    return RouteLeg(
        distance: (map['distance'])?.toDouble(),
        duration: (map['duration'])?.toDouble(),
        summary: map['summary'],
        steps: steps,
        annotation: annotation
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }


    List<Map<String, dynamic>>? stepsMap;
    if(steps != null) {
      stepsMap = List.generate(steps?.length == null? 0: steps!.length, (index) {
        return steps![index].toMap();
      });
    }

    addIfNonNull('distance', this.distance);
    addIfNonNull('duration', this.duration);
    addIfNonNull('summary', this.summary);
    addIfNonNull('steps', stepsMap);
    addIfNonNull('annotation', this.annotation?.toMap());
    return optionsMap;
  }
}


class LegAnnotation {
  final List<double>? distance;
  final List<double>? duration;
  final List<double>? speed;
  final List<MaxSpeed>? maxspeed;
  final List<String>? congestion;
  final List<int>? nodes;
  final List<double>? baseDuration;
  final List<double>? speedLimit;

  LegAnnotation(
      {this.distance,
        this.duration,
        this.speed,
        this.maxspeed,
        this.congestion,
        this.nodes,
        this.baseDuration,
        this.speedLimit});

  static LegAnnotation fromMap(Map<String, dynamic> map) {
    List<MaxSpeed>? maxspeed;
    if (map.containsKey('maxspeed') && map['maxspeed'] != null) {
      List<dynamic> maxspeedMap = map['maxspeed'] as List<dynamic>;
      maxspeed = List.generate(maxspeedMap.length, (index) {
        return MaxSpeed.fromMap(maxspeedMap[index] as Map<String, dynamic>);
      });
    }
    List<String>? congestion;
    if (map.containsKey('congestion') && map['congestion'] != null) {
      List<dynamic> congestionMap = map['congestion'] as List<dynamic>;
      congestion = List.generate(congestionMap.length, (index) {
        return congestionMap[index] as String;
      });
    }
    List<double>? distance;
    if (map.containsKey('distance') && map['distance'] != null) {
      List<dynamic> congestionMap = map['distance'] as List<dynamic>;
      distance = List.generate(congestionMap.length, (index) {
        return (congestionMap[index]).toDouble();
      });
    }
    List<double>? duration;
    if (map.containsKey('duration') && map['duration'] != null) {
      List<dynamic> durationMap = map['duration'] as List<dynamic>;
      duration = List.generate(durationMap.length, (index) {
        return (durationMap[index]).toDouble();
      });
    }
    List<double>? speed;
    if (map.containsKey('speed') && map['speed'] != null) {
      List<dynamic> durationMap = map['speed'] as List<dynamic>;
      speed = List.generate(durationMap.length, (index) {
        return (durationMap[index]).toDouble();
      });
    }
    return LegAnnotation(
        distance: distance,
        duration: duration,
        speed: speed,
        maxspeed: maxspeed,
        congestion: congestion,
        nodes: map['nodes'],
        baseDuration: map['baseDuration'],
        speedLimit: map['speedLimit']);
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    List<Map<String, dynamic>>? maxspeedMap;
    if (maxspeed != null) {
      maxspeedMap = List.generate(
          maxspeed?.length == null ? 0 : maxspeed!.length, (index) {
        return maxspeed![index].toMap();
      });
    }

    addIfNonNull('distance', this.distance);
    addIfNonNull('duration', this.duration);
    addIfNonNull('speed', this.speed);
    addIfNonNull('maxspeed', maxspeedMap);
    addIfNonNull('congestion', this.congestion);
    addIfNonNull('nodes', this.nodes);
    addIfNonNull('baseDuration', this.baseDuration);
    addIfNonNull('speedLimit', this.speedLimit);

    return optionsMap;
  }
}


class MaxSpeed {
  final int? speed;
  final String? unit;
  final bool? unknown;
  final bool? none;

  MaxSpeed({this.speed, this.unit, this.unknown, this.none});

  static MaxSpeed fromMap(Map<String, dynamic> map) {
    return MaxSpeed(
      speed: map['speed'],
      unit: map['unit'],
      unknown: map['unknown'],
      none: map['none'],
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('speed', this.speed);
    addIfNonNull('unit', this.unit);
    addIfNonNull('unknown', this.unknown);
    addIfNonNull('none', this.none);

    return optionsMap;
  }
}
