part of mapmyindia_gl;

class LegStep {
  final double? distance;
  final double? duration;
  final String? geometry;
  final String? name;
  final String? destinations;
  final String? mode;
  final String? pronunciation;
  final String? rotaryName;
  final String? rotaryPronunciation;
  final StepManeuver? maneuver;
  final List<VoiceInstructions>? voiceInstructions;
  final String? drivingSide;
  final double? weight;
  final List<StepIntersection>? intersections;
  final String? exits;
  final String? ref;

  LegStep({
    this.distance,
    this.duration,
    this.geometry,
    this.name,
    this.destinations,
    this.mode,
    this.pronunciation,
    this.rotaryName,
    this.rotaryPronunciation,
    this.maneuver,
    this.voiceInstructions,
    this.drivingSide,
    this.weight,
    this.intersections,
    this.exits,
    this.ref,
  });

  static LegStep fromMap(Map<String, dynamic> map) {
    StepManeuver? maneuver;
    if (map.containsKey('maneuver')) {
      maneuver = StepManeuver.fromMap(map['maneuver']);
    }
    List<VoiceInstructions>? voiceInstructions;
    if (map.containsKey('voiceInstructions')) {
      List<dynamic> voiceInstructionsMap =
      map['voiceInstructions'] as List<dynamic>;
      voiceInstructions = List.generate(voiceInstructionsMap.length, (index) {
        return VoiceInstructions.fromMap(
            voiceInstructionsMap[index] as Map<String, dynamic>);
      });
    }
    List<StepIntersection>? intersections;
    if (map.containsKey('intersections')) {
      List<dynamic> intersectionsMap = map['intersections'] as List<dynamic>;
      intersections = List.generate(intersectionsMap.length, (index) {
        return StepIntersection.fromMap(
            intersectionsMap[index] as Map<String, dynamic>);
      });
    }
    return LegStep(
      distance: (map['distance'])?.toDouble(),
      duration: (map['duration'])?.toDouble(),
      geometry: map['geometry'],
      name: map['name'],
      destinations: map['destinations'],
      mode: map['mode'],
      pronunciation: map['pronunciation'],
      rotaryName: map['rotary_name'],
      rotaryPronunciation: map['rotary_pronunciation'],
      maneuver: maneuver,
      voiceInstructions: voiceInstructions,
      drivingSide: map['driving_side'],
      weight: (map['weight'])?.toDouble(),
      intersections: intersections,
      exits: map['exits'],
      ref: map['ref'],
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    List<Map<String, dynamic>>? voiceInstructionsMap;
    if (voiceInstructions != null) {
      voiceInstructionsMap = List.generate(
          voiceInstructions?.length == null ? 0 : voiceInstructions!.length,
              (index) {
            return voiceInstructions![index].toMap();
          });
    }

    List<Map<String, dynamic>>? intersectionsMap;
    if (intersections != null) {
      intersectionsMap = List.generate(
          intersections?.length == null ? 0 : intersections!.length, (index) {
        return intersections![index].toMap();
      });
    }

    addIfNonNull('distance', this.distance);
    addIfNonNull('duration', this.duration);
    addIfNonNull('geometry', this.geometry);
    addIfNonNull('name', this.name);
    addIfNonNull('destinations', this.destinations);
    addIfNonNull('mode', this.mode);
    addIfNonNull('pronunciation', this.pronunciation);
    addIfNonNull('rotary_name', this.rotaryName);
    addIfNonNull('maneuver', this.maneuver?.toMap());
    addIfNonNull('voiceInstructions', voiceInstructionsMap);
    addIfNonNull('driving_side', drivingSide);
    addIfNonNull('weight', weight);
    addIfNonNull('intersections', intersectionsMap);
    addIfNonNull('ref', ref);
    return optionsMap;
  }
}


class StepManeuver {
  final double? degree;
  final LatLng? location;
  final double? bearingBefore;
  final double? bearingAfter;
  final String? instruction;
  final String? type;
  final String? modifier;
  final int? exit;

  StepManeuver(
      {this.degree,
        this.location,
        this.bearingBefore,
        this.bearingAfter,
        this.instruction,
        this.type,
        this.modifier,
        this.exit});

  static StepManeuver fromMap(Map<String, dynamic> map) {
    LatLng? location;
    if (map.containsKey('location')) {
      List<dynamic> points = map['location'];
      location = LatLng(points[1], points[0]);
    }
    return StepManeuver(
      degree: (map['degree'])?.toDouble(),
      location: location,
      bearingBefore: (map['bearing_before'])?.toDouble(),
      bearingAfter: (map['bearing_after'])?.toDouble(),
      instruction: map['instruction'],
      type: map['type'],
      modifier: map['modifier'],
      exit: map['exit'],
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('degree', degree);
    addIfNonNull('location', this.location?.toJson());
    addIfNonNull('bearing_before', this.bearingBefore);
    addIfNonNull('bearing_after', this.bearingAfter);
    addIfNonNull('instruction', this.instruction);
    addIfNonNull('type', this.type);
    addIfNonNull('modifier', this.modifier);
    addIfNonNull('exit', this.exit);
    return optionsMap;
  }
}


class VoiceInstructions {
  final double? distanceAlongGeometry;
  final String? announcement;
  final String? ssmlAnnouncement;

  VoiceInstructions(
      {this.distanceAlongGeometry, this.announcement, this.ssmlAnnouncement});

  static VoiceInstructions fromMap(Map<String, dynamic> map) {
    return VoiceInstructions(
      distanceAlongGeometry: (map['distanceAlongGeometry'])?.toDouble(),
      announcement: map['announcement'],
      ssmlAnnouncement: map['ssmlAnnouncement'],
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('distanceAlongGeometry', this.distanceAlongGeometry);
    addIfNonNull('announcement', this.announcement);
    addIfNonNull('ssmlAnnouncement ', this.ssmlAnnouncement);
    return optionsMap;
  }
}


class StepIntersection {
  final LatLng? location;
  final List<int>? bearings;
  final List<String>? classes;
  final List<bool>? entry;
  final int? intersectionIn;
  final int? out;
  final List<IntersectionLanes>? lanes;

  StepIntersection(
      {this.location,
        this.bearings,
        this.classes,
        this.entry,
        this.intersectionIn,
        this.out,
        this.lanes});

  static StepIntersection fromMap(Map<String, dynamic> map) {
    LatLng? location;
    if (map.containsKey('location')) {
      List<dynamic> points = map['location'];
      location = LatLng(points[1], points[0]);
    }
    List<IntersectionLanes>? lanes;
    if (map.containsKey('lanes')) {
      List<dynamic> lanesMap = map['lanes'] as List<dynamic>;
      lanes = List.generate(lanesMap.length, (index) {
        return IntersectionLanes.fromMap(
            lanesMap[index] as Map<String, dynamic>);
      });
    }
    List<int>? bearings;
    if(map.containsKey('bearings')) {
      List<dynamic> bearingList = map['bearings'];
      bearings = List.generate(bearingList.length, (index) {
        return bearingList[index] as int;
      });
    }
    List<String>? classes;
    if(map.containsKey('classes')) {
      List<dynamic> classesList = map['classes'];
      classes = List.generate(classesList.length, (index) {
        return classesList[index] as String;
      });
    }
    List<bool>? entry;
    if(map.containsKey('entry')) {
      List<dynamic> entryList = map['entry'];
      entry = List.generate(entryList.length, (index) {
        return entryList[index] as bool;
      });
    }

    return StepIntersection(
        location: location,
        bearings: bearings,
        classes: classes,
        entry: entry,
        intersectionIn: map['in'],
        out: map['out'],
        lanes: lanes
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    List<Map<String, dynamic>>? lanesMap;
    if (lanes != null) {
      lanesMap = List.generate(
          lanes?.length == null ? 0 : lanes!.length,
              (index) {
            return lanes![index].toMap();
          });
    }


    addIfNonNull('location', this.location?.toJson());
    addIfNonNull('bearings', this.bearings);
    addIfNonNull('classes', this.classes);
    addIfNonNull('entry', this.entry);
    addIfNonNull('in', this.intersectionIn);
    addIfNonNull('out', this.out);
    addIfNonNull('lanes', lanesMap);
    return optionsMap;
  }
}

class IntersectionLanes {
  final bool? valid;
  final List<String>? indications;

  IntersectionLanes({this.valid, this.indications});

  static IntersectionLanes fromMap(Map<String, dynamic> map) {
    List<String>? indications;
    if(map.containsKey('indications')) {
      List<dynamic> indicationsList = map['indications'];
      indications = List.generate(indicationsList.length, (index) {
        return indicationsList[index] as String;
      });
    }
    return IntersectionLanes(
        valid: map['valid'],
        indications: indications
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('valid', this.valid);
    addIfNonNull('indications', this.indications);
    return optionsMap;
  }
}