part of mapmyindia_gl;

class MapmyIndiaDirection {
  final LatLng? origin;
  final LatLng? destination;
  final String? originELoc;
  final String? destinationELoc;
  final String profile;
  final String resource;
  final String overview;
  final String geometries;
  final List<LatLng>? waypoint;
  final List<String>? waypointELoc;
  final List<String>? excludes;
  final List<String>? annotations;
  final List<String>? approaches;
  final int? routeType;
  final bool? steps;
  final bool? alternatives;
  final DirectionBearing? bearing;
  final List<int>? waypointIndices;
  final List<String>? waypointNames;
  final List<LatLng>? waypointTargets;
  final bool? bannerInstructions;
  final bool? isSort;
  final bool? lessVerbose;
  final List<double>? radiuses;
  final bool? routeRefresh;
  final String? deviceId;
  final bool? roundaboutExits;
  final String? sessionId;
  final bool? voiceInstructions;
  final String? voiceUnits;

  MapmyIndiaDirection({
    this.origin,
    this.destination,
    this.originELoc,
    this.destinationELoc,
    this.waypoint,
    this.waypointELoc,
    this.excludes,
    this.annotations,
    this.routeType,
    this.steps,
    this.alternatives,
    this.approaches,
    this.bearing,
    this.waypointIndices,
    this.waypointNames,
    this.waypointTargets,
    this.bannerInstructions,
    this.isSort,
    this.lessVerbose,
    this.radiuses,
    this.routeRefresh,
    this.deviceId,
    this.roundaboutExits,
    this.sessionId,
    this.voiceInstructions,
    this.voiceUnits,
    this.profile = DirectionCriteria.PROFILE_DRIVING,
    this.resource = DirectionCriteria.RESOURCE_ROUTE,
    this.overview = DirectionCriteria.OVERVIEW_FULL,
    this.geometries = DirectionCriteria.GEOMETRY_POLYLINE6,
  })  : assert(originELoc != null || origin != null),
        assert(destinationELoc != null || destination != null);

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('originPoint', origin?.toJson());
    addIfNonNull('destinationPoint', destination?.toJson());
    addIfNonNull('originELoc', originELoc);
    addIfNonNull('destinationELoc', destinationELoc);
    if (waypoint != null) {
      optionsMap['waypointPoint'] =
          List.generate(waypoint!.length, (index) => waypoint![index].toJson());
    }
    addIfNonNull('profile', profile);
    addIfNonNull('resource', resource);
    addIfNonNull('overview', overview);
    addIfNonNull('geometries', geometries);
    addIfNonNull('waypointELoc', waypointELoc);
    addIfNonNull('excludes', excludes);
    addIfNonNull('annotations', annotations);
    addIfNonNull('routeType', routeType);
    addIfNonNull('steps', steps);
    addIfNonNull('alternatives', alternatives);
    addIfNonNull('approaches', approaches);
    addIfNonNull('bearing', bearing?.toJson());
    addIfNonNull('waypointIndices', waypointIndices);
    addIfNonNull('waypointNames', waypointNames);
    if (waypointTargets != null) {
      optionsMap['waypointTargets'] = List.generate(
          waypointTargets!.length, (index) => waypointTargets![index].toJson());
    }
    addIfNonNull('bannerInstructions', bannerInstructions);
    addIfNonNull('isSort', isSort);
    addIfNonNull('lessVerbose', lessVerbose);
    addIfNonNull('radiuses', radiuses);
    addIfNonNull('routeRefresh', routeRefresh);
    addIfNonNull('deviceId', deviceId);
    addIfNonNull('roundaboutExits', roundaboutExits);
    addIfNonNull('sessionId', sessionId);
    addIfNonNull('voiceInstructions', voiceInstructions);
    addIfNonNull('voiceUnits', voiceUnits);

    return optionsMap;
  }

  Future<DirectionResponse?> callDirection() async {
    try {
      String? result = await restApi.direction(toJson());
      return result != null
          ? DirectionResponse.fromMap(json.decode(result))
          : null;
    } on PlatformException catch (e) {
      return Future.error(e);
    }
  }
}

class DirectionBearing {
  final double angle;
  final double tolerance;

  DirectionBearing(this.angle, this.tolerance);

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('angle', angle);
    addIfNonNull('tolerance', tolerance);
    return optionsMap;
  }
}
