part of mapmyindia_gl;


class RouteOptions {
  final String? baseUrl;
  final String? deviceID;
  final String? user;
  final String? profile;
  final String? resource;
  final List<String>? coordinates;
  final bool? alternatives;
  final String? language;
  final String? radiuses;
  final String? bearings;
  final bool? lessverbose;
  final String? geometries;
  final String? overview;
  final bool? steps;
  final String? annotations;
  final String? exclude;
  final bool? roundaboutExits;
  final bool? voiceInstructions;
  final bool? bannerInstructions;
  final String? voiceUnits;
  final String? accessToken;
  final String? requestUuid;
  final String? sessionId;
  final String? approaches;
  final String? waypointIndices;
  final String? waypointNames;
  final String? waypointTargets;
  final WalkingOptions? walkingOptions;
  final int? routeType;
  final bool? isSort;
  final bool? routeRefresh;


  RouteOptions(
      {this.baseUrl,
        this.deviceID,
        this.user,
        this.profile,
        this.resource,
        this.coordinates,
        this.alternatives,
        this.language,
        this.radiuses,
        this.bearings,
        this.lessverbose,
        this.geometries,
        this.overview,
        this.steps,
        this.annotations,
        this.exclude,
        this.roundaboutExits,
        this.voiceInstructions,
        this.bannerInstructions,
        this.voiceUnits,
        this.accessToken,
        this.requestUuid,
        this.sessionId,
        this.approaches,
        this.waypointIndices,
        this.waypointNames,
        this.waypointTargets,
        this.walkingOptions,
        this.routeType,
        this.isSort,
        this.routeRefresh});

  static RouteOptions fromJson(Map<String, dynamic> map) {
    WalkingOptions? walkingOptions;
    if(map.containsKey('walkingOptions')) {
      walkingOptions = WalkingOptions.fromMap(map['walkingOptions']);
    }
    List<String>? coordinates;
    if(map.containsKey('coordinates')) {
      List<dynamic> coordinatesList = map['coordinates'];
      coordinates = List.generate(coordinatesList.length, (index) {
        return coordinatesList[index] as String;
      });
    }
    return RouteOptions(
      baseUrl: map['baseUrl'],
      deviceID: map['deviceID'],
      user: map['user'],
      profile: map['profile'],
      resource: map['resource'],
      coordinates: coordinates,
      alternatives: map['alternatives'],
      language: map['language'],
      radiuses: map['radiuses'],
      bearings: map['bearings'],
      lessverbose: map['lessverbose'],
      geometries: map['geometries'],
      overview: map['overview'],
      steps: map['steps'],
      annotations: map['annotations'],
      exclude: map['exclude'],
      roundaboutExits: map['roundabout_exits'],
      voiceInstructions: map['voice_instructions'],
      bannerInstructions: map['banner_instructions'],
      voiceUnits: map['voice_units'],
      accessToken: map['access_token'],
      requestUuid: map['uuid'],
      sessionId: map['sessionId'],
      approaches: map['approaches'],
      waypointIndices: map['waypoints'],
      waypointNames: map['waypoint_names'],
      waypointTargets: map['waypoint_targets'],
      walkingOptions: walkingOptions,
      routeType: map['routeType'],
      isSort: map['isSort'],
      routeRefresh: map['routeRefresh'],
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('baseUrl', this.baseUrl);
    addIfNonNull('deviceID', this.deviceID);
    addIfNonNull('user', this.user);
    addIfNonNull('profile', this.profile);
    addIfNonNull('resource', this.resource);
    addIfNonNull('coordinates', this.coordinates);
    addIfNonNull('alternatives', this.alternatives);
    addIfNonNull('language', this.language);
    addIfNonNull('radiuses', this.radiuses);
    addIfNonNull('bearings', this.bearings);
    addIfNonNull('lessverbose', this.lessverbose);
    addIfNonNull('geometries', this.geometries);
    addIfNonNull('overview', this.overview);
    addIfNonNull('steps', this.steps);
    addIfNonNull('annotations', this.annotations);
    addIfNonNull('exclude', this.exclude);
    addIfNonNull('roundabout_exits', this.roundaboutExits);
    addIfNonNull('voice_instructions', this.voiceInstructions);
    addIfNonNull('banner_instructions', this.bannerInstructions);
    addIfNonNull('voice_units', this.voiceUnits);
    addIfNonNull('access_token', this.accessToken);
    addIfNonNull('uuid', this.requestUuid);
    addIfNonNull('sessionId', this.sessionId);
    addIfNonNull('approaches', this.approaches);
    addIfNonNull('waypoints', this.waypointIndices);
    addIfNonNull('waypoint_names', this.waypointNames);
    addIfNonNull('waypoint_targets', this.waypointTargets);
    addIfNonNull('walkingOptions', this.walkingOptions?.toMap());
    addIfNonNull('routeType', this.routeType);
    addIfNonNull('isSort', this.isSort);
    addIfNonNull('routeRefresh', this.routeRefresh);

    return optionsMap;
  }
}


class WalkingOptions {
  final double? walkingSpeed;
  final double? walkwayBias;
  final double? alleyBias;

  WalkingOptions(
      {this.walkingSpeed, this.walkwayBias, this.alleyBias});

  static WalkingOptions fromMap(Map<String, dynamic> map) {
    return WalkingOptions(
      walkingSpeed: map['walking_speed'].toDouble(),
      walkwayBias: map['walkway_bias'].toDouble(),
      alleyBias: map['alley_bias'].toDouble(),
    );
  }

  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    addIfNonNull('walking_speed', this.walkingSpeed);
    addIfNonNull('walkway_bias', this.walkwayBias);
    addIfNonNull('alley_bias ', this.alleyBias);
    return optionsMap;
  }
}